"""
Command line utility to extract basic statistics from gpx file(s)
"""

import sys as mod_sys
import logging as mod_logging
import math as mod_math
import argparse as mod_argparse

import gpxpy as mod_gpxpy
import gpxpy.gpx as mod_gpx

from typing import *

KM_TO_MILES = 0.621371
M_TO_FEET = 3.28084


def format_time(time_s: float, seconds: bool) -> str:
    if not time_s:
        return 'n/a'
    elif seconds:
        return str(int(time_s))
    else:
        minutes = mod_math.floor(time_s / 60.)
        hours = mod_math.floor(minutes / 60.)
        return '%s:%s:%s' % (str(int(hours)).zfill(2), str(int(minutes % 60)).zfill(2), str(int(time_s % 60)).zfill(2))


def format_long_length(length: float, miles: bool) -> str:
    if miles:
        return '{:.3f}miles'.format(length / 1000. * KM_TO_MILES)
    else:
        return '{:.3f}km'.format(length / 1000.)


def format_short_length(length: float, miles: bool) -> str:
    if miles:
        return '{:.2f}ft'.format(length * M_TO_FEET)
    else:
        return '{:.2f}m'.format(length)


def format_speed(speed: float, miles: bool) -> str:
    if not speed:
        speed = 0
    if miles:
        return '{:.2f}mph'.format(speed * KM_TO_MILES * 3600. / 1000.)
    else:
        return '{:.2f}m/s = {:.2f}km/h'.format(speed, speed * 3600. / 1000.)


def print_gpx_part_info(gpx_part: Union[mod_gpx.GPX, mod_gpx.GPXTrack, mod_gpx.GPXTrackSegment], indentation: str='    ', miles: bool = False, seconds: bool = False) -> None:
    """
    gpx_part may be a track or segment.
    """
    length_2d = gpx_part.length_2d()
    length_3d = gpx_part.length_3d()
    print('%sLength 2D: %s' % (indentation, format_long_length(length_2d or 0, miles)))
    print('%sLength 3D: %s' % (indentation, format_long_length(length_3d, miles)))

    moving_data = gpx_part.get_moving_data()
    if moving_data:
        print('%sMoving time: %s' % (indentation, format_time(moving_data.moving_time, seconds)))
        print('%sStopped time: %s' % (indentation, format_time(moving_data.stopped_time, seconds)))
        #print('%sStopped distance: %s' % (indentation, format_short_length(stopped_distance)))
        print('%sMax speed: %s' % (indentation, format_speed(moving_data.max_speed, miles)))
        print('%sAvg speed: %s' % (indentation, format_speed(moving_data.moving_distance / moving_data.moving_time, miles) if moving_data.moving_time > 0 else "?"))

    uphill, downhill = gpx_part.get_uphill_downhill()
    print('%sTotal uphill: %s' % (indentation, format_short_length(uphill, miles)))
    print('%sTotal downhill: %s' % (indentation, format_short_length(downhill, miles)))

    start_time, end_time = gpx_part.get_time_bounds()
    print('%sStarted: %s' % (indentation, start_time))
    print('%sEnded: %s' % (indentation, end_time))

    points_no = len(list(gpx_part.walk(only_points=True)))
    print('%sPoints: %s' % (indentation, points_no))

    if points_no > 0:
        distances: List[float] = []
        previous_point = None
        for point in gpx_part.walk(only_points=True):
            if previous_point:
                distance = point.distance_2d(previous_point)
                distances.append(distance)
            previous_point = point
        print('%sAvg distance between points: %s' % (indentation, format_short_length(sum(distances) / len(list(gpx_part.walk())), miles)))

    print('')


def print_gpx_info(gpx: mod_gpx.GPX, gpx_file: str, miles: bool, seconds: bool, only_track: bool) -> None:
    print('File: %s' % gpx_file)

    if gpx.name:
        print('  GPX name: %s' % gpx.name)
    if gpx.description:
        print('  GPX description: %s' % gpx.description)
    if gpx.author_name:
        print('  Author: %s' % gpx.author_name)
    if gpx.author_email:
        print('  Email: %s' % gpx.author_email)
    print(f'    Waypoints: {len(gpx.waypoints)}')
    print(f'    Routes: {len(gpx.routes)}')

    print_gpx_part_info(gpx, miles=miles, seconds=seconds)

    if only_track:
        return

    for track_no, track in enumerate(gpx.tracks):
        for segment_no, segment in enumerate(track.segments):
            print('    Track #%s, Segment #%s' % (track_no, segment_no))
            print_gpx_part_info(segment, indentation='        ', miles=miles, seconds=seconds)


def run(gpx_files: List[str], miles: bool, seconds: bool, only_track: bool) -> None:
    if not gpx_files:
        print('No GPX files given')
        mod_sys.exit(1)

    for gpx_file in gpx_files:
        try:
            gpx = mod_gpxpy.parse(open(gpx_file))
            print_gpx_info(gpx, gpx_file, miles, seconds, only_track)
        except Exception as e:
            mod_logging.exception(e)
            print('Error processing %s' % gpx_file)
            mod_sys.exit(1)


def main() -> None:
    parser = mod_argparse.ArgumentParser(usage='%(prog)s [-s] [-m] [-d] [file ...]',
        description='Command line utility to extract basic statistics from gpx file(s)')
    parser.add_argument('-t', '--track', action='store_true', help='Only root track')
    parser.add_argument('-s', '--seconds', action='store_true', help='print times as N seconds, rather than HH:MM:SS')
    parser.add_argument('-m', '--miles', action='store_true', help='print distances and speeds using miles and feet')
    parser.add_argument('-d', '--debug', action='store_true', help='show detailed logging')
    args, gpx_files = parser.parse_known_args()
    seconds = args.seconds
    miles = args.miles
    debug = args.debug
    only_track = args.track

    if debug:
        mod_logging.basicConfig(level=mod_logging.DEBUG, format='%(asctime)s %(name)-12s %(levelname)-8s %(message)s')
    run(gpx_files=gpx_files, miles=miles, seconds=seconds, only_track=only_track)


"""
Command line utility to extract basic statistics from gpx file(s)
"""

import sys as mod_sys
import logging as mod_logging
import math as mod_math
import argparse as mod_argparse
from . import common

import gpxpy as mod_gpxpy
import gpxpy.gpx as mod_gpx

from typing import *

def main() -> None:
    parser = mod_argparse.ArgumentParser(description='Simplify tracks using the Ramer-Douglas-Peucker algorithm')
    parser.add_argument('-d', '--distance', type=float, default=10, help='Max distance')
    parser.add_argument('gpx_files', metavar='gpx', type=str, default='', nargs=1, help='GPX file')

    args = parser.parse_args()
    gpx_file = args.gpx_files[0]
    distance = args.distance

    with open(gpx_file) as f:
        g = mod_gpxpy.parse(f)
    
    g.simplify(max_distance=distance)

    with open(common.prefix_filename(gpx_file, "_simplified"), "w") as f:
        f.write(g.to_xml())


import argparse
import gpxpy.gpx as gpx
import gpxpy
import datetime
import os.path as path
import datetime

from . import common

from typing import *

def get_time(g: gpx.GPX) -> Optional[datetime.datetime]:
    for t in g.tracks:
        for s in t.segments:
            for pt in s.points:
                if pt.time:
                    return pt.time
    return None

def main() -> None:
    parser = argparse.ArgumentParser(description='Merge GPX files')
    parser.add_argument('gpx_files', metavar='gpx', type=str, default='', nargs='*', help='GPX file')
    parser.add_argument('-o', '--output', metavar='F', type=str, default='merged.gpx', help='Output GPX file')
    parser.add_argument('-t', '--time', action='store_true', help='Sort by time')
    args = parser.parse_args()

    gpx_files = args.gpx_files
    out_file = args.output
    sort_by_time = args.time

    keep_extensions = True

    if not gpx_files:
        print("Nothing to do")
        return

    gpxs: List[gpx.GPX] = []

    for gpx_file in gpx_files:
        print(f"Reading {gpx_file}")
        gpxs.append(gpxpy.parse(open(gpx_file)))

    if sort_by_time:
        print("Sorting by time")
        gpxs = sorted(gpxs, key=get_time)

    base_gpx: Optional[gpx.GPX] = None
    for g in gpxs:
        if base_gpx:
            if len(g.nsmap) != len(base_gpx.nsmap):
                keep_extensions = False
            else:
                for key in base_gpx.nsmap:
                    if not key in g.nsmap or g.nsmap[key] != base_gpx.nsmap[key]:
                        keep_extensions = False
        else:
            base_gpx = g.clone()
            base_gpx.tracks = []
            base_gpx.routes = []
        
        for track in g.tracks:
            print("adding track")
            base_gpx.tracks.append(track)
        for route in g.routes:
            base_gpx.routes.append(route)
        
    if base_gpx:
        if not keep_extensions:
            print("Removing extensions")
            common.clean_extensions(base_gpx)
    
        with open(out_file, "w") as f:
            f.write(base_gpx.to_xml())


import argparse
import gpxpy.gpx as gpxpy
import gpxpy as gpx_parser

from . import common

from typing import *

def main() -> None:
    parser = argparse.ArgumentParser(description='Clean GPX tracks')
    parser.add_argument('-e', '--extensions', action='store_true', help='Remove extensions')
    parser.add_argument('-t', '--time', action='store_true', help='Remove time')
    parser.add_argument('-x', '--elevations', action='store_true', help='Remove extensions')
    parser.add_argument('-r', '--routes', action='store_true', help='Remove routes')
    parser.add_argument('-tr', '--tracks', action='store_true', help='Remove tracks')
    parser.add_argument('-a', '--author', action='store_true', help='Remove author data')
    parser.add_argument('-w', '--waypoints', action='store_true', help='Waypoints')
    parser.add_argument('-o', '--output', metavar='F', type=str, default='clean.gpx', help='Output GPX file')
    args, gpx_files = parser.parse_known_args()

    extensions: bool = args.extensions
    time: bool = args.time
    elevations: bool = args.elevations
    routes: bool = args.routes
    tracks: bool = args.tracks
    author: bool = args.author
    waypoints: bool = args.waypoints
    output = args.output

    for gpx_file in gpx_files:
        with open(gpx_file) as f:
            g = gpx_parser.parse(f)
        if extensions:
            common.clean_extensions(g)
        if time:
            g.time = None
            g.remove_time()
        if elevations:
            g.remove_elevation()
        if routes:
            g.routes = []
        if tracks:
            g.tracks = []
        if author:
            g.author_name = None
            g.author_email = None
            g.author_link = None
            g.author_link_text = None
            g.author_link_type = None
        if waypoints:
            g.waypoints = []

    if not output:
        output = common.prefix_filename(gpx_file, "clean_")

    with open(output, "w") as f:
        f.write(g.to_xml())




import gpxpy.gpx as gpx
import os.path as path
import re

from typing import *

def parse_duration(s: str) -> int:
    all = re.findall("(\d+)(\w+)", s)
    if not all:
        return -1
    i = int(all[0][0])
    unit = all[0][1]
    if unit == "s":
        return i
    elif unit == "m":
        return i * 60 * 1000
    elif unit == "h":
        return i * 60 * 60 * 1000
    else:
        return -1

def clean_extensions(g: gpx.GPX) -> None:
    g.extensions = []
    for rte in g.routes:
        rte.extensions = []
        for pt in rte.points:
            pt.extensions = []
    for t in g.tracks:
        t.extensions = []
        for s in t.segments:
            s.extensions = []
            for p in s.points:
                p.extensions = []

def prefix_filename(fn: str, prefix: str) -> str:
    dir, fn = path.split(fn)
    return path.join(dir, prefix + fn)


import argparse
import gpxpy.gpx as gpxpy
import gpxpy as gpx_parser
import datetime
import sys

from . import common

from typing import *

DATE_FORMAT = "%Y-%m-%d"
LONG_TIME_FORMAT = '%Y-%m-%d %H:%M:%S'
TIME_FORMAT = '%Y%m%d%H%M%S'

def need_split_by_days(t1: datetime.datetime, t2: datetime.datetime, seconds: int) -> bool:
    return t1.strftime(DATE_FORMAT) != t2.strftime(DATE_FORMAT)

def need_split_by_time_interval(t1: datetime.datetime, t2: datetime.datetime, seconds: int) -> bool:
    return abs((t2 - t1).total_seconds()) > seconds

def save_gpx(gpg: gpxpy.GPX) -> None:
    pass

def process(gpx_file: str, split_by_days: bool, time_interval: str) -> None:
    print(f"Opening {gpx_file}")
    with open(gpx_file) as f:
        gpx = gpx_parser.parse(f)

    split_func: Callable[[datetime.datetime, datetime.datetime, int], bool]
    interval = 0
    if split_by_days:
        split_func = need_split_by_days
    elif time_interval:
        split_func = need_split_by_time_interval
        interval = common.parse_duration(time_interval)
    else:
        print("Specify way to split GPX file")
        sys.exit(1)

    times: List[datetime.datetime] = []

    for track in gpx.tracks:
        for segment in track.segments:
            for pt in segment.points:
                if pt.time:
                    times.append(pt.time)

    if not times:
        print("No times in track")
        sys.exit(0)

    times.sort()
    
    split_times: List[datetime.datetime] = [times[0] - datetime.timedelta(seconds=1)]
    for n, time in enumerate(times):
        if n > 0:
            prev_time = times[n-1]
            if split_func(prev_time, time, interval):
                split_times.append(time)
    split_times.append(times[-1] + datetime.timedelta(seconds = 1))

    #print(f"Split times: {split_times}")
    for n, time in enumerate(split_times):
        if n > 0:
            gpx_interval = gpx.clone()
            time_from = split_times[n-1]
            time_to = time
            print(f"Interval time {time_from.strftime(LONG_TIME_FORMAT)} - {time_to.strftime(LONG_TIME_FORMAT)}")
            for track in gpx_interval.tracks:
                for segment in track.segments:
                    def fltr(pt: gpxpy.GPXTrackPoint) -> bool:
                        return bool(pt.time and time_from <= pt.time and pt.time < time_to)
                    segment.points = list(filter(fltr, segment.points))
            gpx_interval.remove_empty()
            gpx_interval.remove_empty()
            gpx_interval.remove_empty()

            fn = common.prefix_filename(gpx_file, time_from.strftime(TIME_FORMAT))
            with open(fn, "w") as f:
                f.write(gpx_interval.to_xml())
                print(f"Saved {fn}")

def main() -> None:
    parser = argparse.ArgumentParser(description='Split GPX file')
    parser.add_argument('-d', '--days', action='store_true', default=False, help='How many days')
    parser.add_argument('-t', '--time', type=str, default="", help='Split by time intervals (for exampe 1m, 2s, 3h)')
    parser.add_argument('gpx_files', metavar='gpx', type=str, default='', nargs='*', help='GPX file')
    args = parser.parse_args()

    gpx_files = args.gpx_files
    split_by_days = args.days
    time_interval = args.time

    for gpx_file in gpx_files:
        process(gpx_file, split_by_days = split_by_days, time_interval=time_interval)


#!/usr/bin/env python3
#
# Copyright 2020 Tomo Krajina
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import distutils.core as mod_distutilscore

mod_distutilscore.setup(
    name='gpx-cmd-tools',
    version='0.0.1',
    description='Set of GPX command-line utilities',
    license='Apache License, Version 2.0',
    author='Tomo Krajina',
    author_email='tkrajina@gmail.com',
    url='https://github.com/tkrajina/gpx-cmd-tools',
    packages=['gpxtools', ],
    classifiers=[
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
    ],
    scripts=[ 'gpxclean', 'gpxinfo', 'gpxmerge', 'gpxsimplify', 'gpxsplitter' ],
    install_requires=['gpxpy'],
)
